	processor 6502
	include "vcs.h"
	include "macro.h"

	;leave both 0 for NTSC
	;don't set both to 1...
PAL60 equ 1
PAL50 equ 0


	;game constants...
	IF PAL50
defscrollspeed	equ 43
xaccel			equ 5
animspeed		equ 48
title_speed		equ 36
platinterval	equ 76
vislines		equ 228
vblank_wait		equ 9
overscan_wait	equ 7
	ELSE
defscrollspeed	equ 36
xaccel			equ 4
animspeed		equ 40
title_speed		equ 30
platinterval	equ 74
vislines		equ 192
vblank_wait		equ 1
overscan_wait	equ 1
	ENDIF


	;colors...
	IF PAL60 || PAL50
scorecolubk			equ $06
p0colu				equ $62
p1colu				equ $92		;b2 is a closer match, but it doesn't work...
free_pf_colu		equ $01
p0bonuscolu			equ $6a
p1bonuscolu			equ $9a
title_pic_colu		equ $0e
title_border_colu	equ $06
icon_colu			equ $0e
	ELSE
scorecolubk			equ $06
p0colu				equ $42
p1colu				equ $82
free_pf_colu		equ $01
p0bonuscolu			equ $4a
p1bonuscolu			equ $8a
title_pic_colu		equ $0e
title_border_colu	equ $06
icon_colu			equ $0e
	ENDIF


	;more game constants...
platheight		equ 8
playerheight	equ 17
playerwidth		equ 8
sxoffset		equ 4
pxoffset		equ 12
balltop			equ 5
gravacc			equ 1
numberlines		equ 8
scorelines		equ 22				;numberlines plus a little padding
mainlines		equ [vislines-scorelines]
maxplats		equ [mainlines/platinterval+1]		;max on-screen platforms
													;must be less than the number of buffers!

ai_fire_line	equ mainlines/2						;the ai will use power-ups above this line
dieline			equ mainlines+playerheight+4		;players die above this line
deadline		equ 240								;dead players are marked with this
deadtime		equ 23								;dead players are disabled for this long
maxjump			equ 7
xdecel			equ 2
nmodes			equ 8
iconheight		equ 7
maxpower		equ 5*16
	;His name sounds good in your ear,
	;but when you hear it, you musn't fear!
	;For his name can be said by anyooooooone!

	;power-up states
state_normal	equ 0
state_fast		equ 1
state_break		equ 2
state_wayfast	equ 3
state_ghost		equ 4
state_swap		equ 5



	seg.u vars
	org $80

linebuf0		.ds 14
randseed		.ds 2

linebuf1		.ds 14
holeindex		.ds 1
topline			.ds 1		;bottom line of the topmost platform
select_switch	equ topline	;title screen only

linebuf2		.ds 14
scrollspeed		.ds 1
scrollcount		.ds 1

linebuf3		.ds 14
topbuf			.ds 1		;indexed value - points to the buffer /before/ the top one,
							;i.e. the one currently off-screen
captures		.ds 1
bonus			.ds 1
ai_memory		.ds 1
sound			.ds 1
subsound		.ds 1
sound_timer		.ds 1
bg_color		.ds 1
swchb_bak		.ds 1
pause			.ds 1

	;-- data for the platform being drawn --
platform_data	equ *
liney			.ds 1		;line equal to the bottom of this platform
linebufptr		.ds 2		;pointer to the buffer to get the next data set from
platcolor		.ds 1		;COLUPF
linegfx0		.ds 1		;left-side data for PF0-2
linegfx1		.ds 1
linegfx2		.ds 1
linegfx3		.ds 1		;right-side 
linegfx4		.ds 1
linegfx5		.ds 1
p0bufptr		.ds 2
p1bufptr		.ds 2

scratch			.ds 2			;used outside the kernel
colubonus		equ scratch		;used in the kernel only
moscratch		.ds 2
ballptr			equ moscratch

	;-- misc data used outside the kernel --
	;(destroyed every frame)
stage			equ platcolor
ai_control		equ linegfx1
p1_input		equ linegfx2
ai_fire			equ linegfx3
p1_fire			equ linegfx4
sound_request_low	equ p0bufptr
sound_request_high	equ p1bufptr

title_timer_h	equ moscratch
title_timer_l	equ moscratch+1

	;-- pointers for the score display --
digit0			equ linegfx0
digit1			equ linegfx2
digit2			equ linegfx4
digit3			equ p0bufptr
digit4			equ p1bufptr
digit5 			equ scratch

	;-- player data --
p0x				.ds 1
p1x				.ds 1
p0y				.ds 1		;must follow pNx
p1y				.ds 1
p0fracx			.ds 1
p1fracx			.ds 1
p0dx			.ds 1
p1dx			.ds 1
p0dy			.ds 1		;must follow pNdx
p1dy			.ds 1
p0fracy			.ds 1
p1fracy			.ds 1
p0ancnt			.ds 1
p1ancnt			.ds 1
p0frame			.ds 1		;bit 6 is used for reflection, 7 for collisions
p1frame			.ds 1
p0moveflags		.ds 1
p1moveflags		.ds 1
p0jumpcnt		.ds 1		;also used as a game-over flag
p1jumpcnt		.ds 1
p0state			.ds 1
p1state			.ds 1
p0timer			.ds 1
p1timer			.ds 1

end_marker		equ p0jumpcnt

preserve		equ *		;memory above here is not cleared on a reset

p0lscore		.ds 1
p1lscore		.ds 1
p0hscore		.ds 1		;doubles as bonus counter
p1hscore		.ds 1

hiscore_l		.ds 1
mode			.ds 1
hiscore_h		.ds 1		;split by 1 byte because the player scores are that way too


	;temporary data that can be destroyed by the stack
	;(5 bytes of stack needed)
	;(only 3 for most of the program though)
p0gfxptr		.ds 2
p1gfxptr		.ds 2

	echo $100-*
	echo "bytes left"


	include "macro_functions.asm"
	include "title.asm"		;also a macro


	;*****************************
	;** ACTUAL CODE STARTS HERE **
	;*****************************
	seg main
	org $f000

start:
	mac_start_game

reset:
	mac_reset_game

	mac_display_title

	mac_init

frame_start:
	;*******************
	;* vsync - 3 lines *
	;*******************
	jsr do_vsync

	;*********************
	;* vblank - 37 lines *
	;*********************
	mac_do_vblank


	;******************************
	;* visible screen - 192 lines *
	;******************************
	mac_display_score
	;next few lines will be in scorecolubk

	mac_kernel_prep
	;this returns on cycle 10

	lda bg_color			;3
	sta COLUBK				;3

	mac_do_player_ref		;39

	lda (scratch,x)			;6

	txa						;2
	sec						;2
	ldx #mainlines			;2 = 67

	ldy linegfx0			;3

	sta HMOVE				;finishes on 73 ----> no lame black line

	include "kernel.asm"



	;***********************
	;* overscan - 30 lines *
	;***********************
start_overscan:
	mac_do_overscan





	include "subroutines.asm"

	include "sound.asm"

	org $fc22
	include "title_gfx.asm"

	org $fd00
	include "data.asm"

	org $fda6			;~p0colu+platheight
	include "data_high.asm"

	org $fe00
	include "sound_data.asm"

	org $fe96			;<--- oh, the craziness!
	include "player_gfx_1.asm"

	org $ff00
	include "score_gfx.asm"
	include "stage_data.asm"

	org $ff96
	include "player_gfx_2.asm"


	;*********************
	;* interrupt vectors *
	;*********************
	org $fffa
	.word start
	.word start
	.word waste_lines

	END